@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import logging
 import re
 import subprocess
 
@@ -19,14 +20,15 @@ from osc_lib.command import command
 
 ##
 
-def _add_port_arguments(ports):
-    arg = ""
-    for port in ports:
-        if port != 22:
-            arg += "--add-port %d " % (port)
-    return arg
-
 class _AddPort(command.Command):
+
+    def add_port_arguments(self, ports):
+        arg = ""
+        for port in ports:
+            if port != 22:
+                arg += "--add-port %d " % (port)
+        return arg
+
     def get_parser(self, prog_name):
         parser = super(_AddPort, self).get_parser(prog_name)
         parser.add_argument(
@@ -35,6 +37,7 @@ class _AddPort(command.Command):
             help='Destination port (allow multiple times, default: [22]).',
         )
         return parser
+
 ##
 
 class _Flavor(command.Command):
@@ -49,6 +52,36 @@ class _Flavor(command.Command):
 ##
 
 class _Vanilla(command.Command):
+
+    def _insert_verbosity_option(self, cmd):
+        verbosity = ''
+        verbose_level = self.app.options.verbose_level
+        if verbose_level == 0:
+            verbosity = '-q'
+        elif verbose_level > 1:
+            verbosity = '-%s' % ('v' * (verbose_level-1))
+        return re.sub(r'(openstack \S+)', r'\1 %s' % (verbosity), cmd)
+
+    logger = logging.getLogger(__name__)
+
+    def check_call(self, cmd, silent=''):
+        cmd = self._insert_verbosity_option(cmd)
+        if self.app.options.verbose_level < 2:
+            cmd += ' %s' % (silent)
+        self.logger.info("-- _Vanilla.check_call ------\n%s\n--------" % cmd)
+        subprocess.check_call('sleep 1 && ' + cmd, shell=True)
+        return
+
+    def check_calls(self, cmds):
+        for cmd in cmds:
+            self.check_call(cmd)
+        return
+
+    def check_output(self, cmd):
+        cmd = self._insert_verbosity_option(cmd)
+        self.logger.info("-- _Vanilla.check_output ------\n%s\n--------" % cmd)
+        return subprocess.check_output('sleep 1 && ' + cmd, shell=True)
+
     def get_parser(self, prog_name):
         parser = super(_Vanilla, self).get_parser(prog_name)
         parser.add_argument(
@@ -63,9 +96,10 @@ class AllowMe(_Vanilla, _AddPort):
 
     def take_action(self, parsed_args):
         server = parsed_args.server
-        subprocess.check_call("sleep 1 && openstack security group create --description 'for %s' %s > /dev/null && openstack server add security group %s %s" % (server, server, server, server), shell=True)
+        self.check_call("openstack security group create --format yaml --description 'for %s' %s" % (server, server), '> /dev/null')
+        self.check_call('openstack server add security group %s %s' % (server, server))
         for port in parsed_args.ports:
-            subprocess.check_call("sleep 1 && openstack security group rule create --proto tcp --dst-port %d --remote-ip `openstack vanilla show my ip` %s > /dev/null" % (port, server), shell=True)
+            self.check_call('openstack security group rule create --format yaml --proto tcp --dst-port %d --remote-ip `openstack vanilla show my ip %s` %s' % (port, server, server), '> /dev/null')
         return
 
 ##
@@ -89,7 +123,7 @@ class Create(_Vanilla, _Flavor, _AddPort):
         )
         return parser
     
-    def _is_float(s):
+    def _is_float(self, s):
         try:
             float(s)
         except ValueError:
@@ -100,11 +134,15 @@ class Create(_Vanilla, _Flavor, _AddPort):
     def take_action(self, parsed_args):
         server = parsed_args.server
         volume = parsed_args.volume
-        if volume and _is_float(volume):
-            volume = subprocess.check_output("sleep 1 && openstack volume create --description 'for %s' --size %s --format value --column id %s" % (server, volume, server), shell=True).decode()
-        subprocess.check_call("sleep 1 && openstack server create --flavor %s --image %s --key-name %s --wait %s > /dev/null && openstack vanilla allow me %s %s && openstack vanilla give ip %s" % (parsed_args.flavor, parsed_args.image, parsed_args.key_name, server, _add_port_arguments(parsed_args.ports), server, server), shell=True)
+        if volume and self._is_float(volume):
+            volume = self.check_output("openstack volume create %s --description 'for %s' --size %s --format value --column id" % (server, server, volume)).decode().strip()
+        self.check_call('openstack server create --format yaml --flavor %s --image %s --key-name %s --wait %s' % (parsed_args.flavor, parsed_args.image, parsed_args.key_name, server), '> /dev/null')
+        self.check_calls([
+            'openstack vanilla allow me %s %s' % (self.add_port_arguments(parsed_args.ports), server),
+            'openstack vanilla give ip %s' % (server),
+        ])
         if volume:
-            subprocess.check_call("sleep 1 && openstack server add volume %s %s" % (server, volume), shell=True)
+            self.check_call('openstack server add volume %s %s' % (server, volume))
         return
 
 ##
@@ -116,7 +154,7 @@ class Delete(_Vanilla):
         parser = super(Delete, self).get_parser(prog_name)
         parser.add_argument(
             '--delete-volumes', action='store_true', 
-            help="Delete the attached volumes if exist",
+            help='Delete the attached volumes if exist',
         )
         return parser
     
@@ -125,11 +163,16 @@ class Delete(_Vanilla):
         volumes = parsed_args.delete_volumes
         server_id = False
         if volumes:
-            volumes = subprocess.check_output("sleep 1 && openstack server show %s --format value --column volumes_attached" % (server), shell=True).decode()
-        subprocess.check_call("ssh-keygen -R `openstack vanilla show ip %s` && openstack vanilla take ip %s && openstack vanilla deny us %s && openstack server delete --wait %s" % (server, server, server, server), shell=True)
+            volumes = self.check_output('openstack server show %s --format value --column volumes_attached' % (server)).decode().strip()
+        self.check_call('ssh-keygen -R `openstack vanilla show ip %s`' % (server), '2> /dev/null')
+        self.check_calls([
+            'openstack vanilla take ip %s' % (server),
+            'openstack vanilla deny us %s' % (server),
+            'openstack server delete --wait %s' % (server)
+        ])
         if volumes:
             for volume in re.findall("'([^']+)'", volumes):
-                subprocess.check_call("sleep 1 && openstack volume delete %s" % (volume), shell=True)
+                self.check_call('openstack volume delete %s' % (volume))
         return
 
 ##
@@ -140,11 +183,14 @@ class DenyUs(_Vanilla):
     def take_action(self, parsed_args):
         server = parsed_args.server
         try:
-            subprocess.check_call("sleep 1 && openstack security group show %s > /dev/null 2>&1" %(server), shell=True)
+            self.check_call('openstack security group show --format yaml %s' % (server), '> /dev/null 2>&1')
         except:
             pass
         else:
-            subprocess.check_call("sleep 1 && openstack server remove security group %s %s && openstack security group delete %s" % (server, server, server), shell=True)
+            self.check_calls([
+                'openstack server remove security group %s %s' % (server, server),
+                'openstack security group delete %s' % (server)
+            ])
         return
 ##
 
@@ -153,9 +199,9 @@ class GiveIP(_Vanilla):
 
     def take_action(self, parsed_args):
         server = parsed_args.server
-        ip = subprocess.check_output("openstack vanilla show ip %s" % (server), shell=True).decode().strip()
+        ip = self.check_output('openstack vanilla show ip %s' % (server)).decode().strip()
         if re.search('=', ip):
-            subprocess.check_call("sleep 1 && openstack server add floating ip %s `openstack floating ip create public --format value --column 'floating_ip_address'`" % (server), shell=True)
+            self.check_call('openstack server add floating ip %s `openstack floating ip create public --format value --column floating_ip_address`' % (server))
         return
 
 ##
@@ -165,7 +211,8 @@ class Resize(_Vanilla, _Flavor):
 
     def take_action(self, parsed_args):
         server = parsed_args.server
-        subprocess.check_call("sleep 1 && openstack server resize --flavor %s --wait %s > /dev/null && openstack server resize confirm %s" % (parsed_args.flavor, server, server), shell=True)
+        self.check_call('openstack server resize --flavor %s --wait %s' % (parsed_args.flavor, server), '> /dev/null')
+        self.check_call('openstack server resize confirm %s' % (server))
         return
 
 ##
@@ -183,13 +230,18 @@ class Shelve(_Vanilla):
 
     def take_action(self, parsed_args):
         server = parsed_args.server
-        images =  subprocess.check_output("sleep 1 && openstack image list --format value --column ID --property instance_uuid=`openstack vanilla show id %s`" % (server), shell=True).decode().strip().split()
-        subprocess.check_call("ssh-keygen -R `openstack vanilla show ip %s` && openstack vanilla take ip %s && openstack vanilla deny us %s && openstack server shelve %s > /dev/null" % (server, server, server, server), shell=True)
+        images =  self.check_output('openstack image list --format value --column ID --property instance_uuid=`openstack vanilla show id %s`' % (server)).decode().strip().split()
+        self.check_call('ssh-keygen -R `openstack vanilla show ip %s`' % (server), '2> /dev/null')
+        self.check_calls([
+            'openstack vanilla take ip %s' % (server),
+            'openstack vanilla deny us %s' % (server),
+            'openstack server shelve %s' % (server)
+        ])
         if not parsed_args.keep_images:
-            while 'SHELVED_OFFLOADED' != subprocess.check_output("openstack vanilla show status %s" % (server), shell=True).decode().strip():
-                subprocess.check_call("sleep 1", shell=True)
+            while 'SHELVED_OFFLOADED' != self.check_output('openstack vanilla show status %s' % (server)).decode().strip():
+                self.check_call('sleep 4')
             for image in images:
-                subprocess.check_call("sleep 1 && openstack image delete %s > /dev/null 2>&1" % (image), shell=True)
+                self.check_call('openstack image delete %s' % (image))
         return
 
 ##
@@ -198,7 +250,7 @@ class ShowID(_Vanilla):
     """Show the ID of the vanilla server."""
 
     def take_action(self, parsed_args):
-        subprocess.check_call("sleep 1 && openstack server show --format value --column id %s" % (parsed_args.server), shell=True)
+        self.check_call('openstack server show --format value --column id %s' % (parsed_args.server))
         return
 
 ##
@@ -207,16 +259,16 @@ class ShowIP(_Vanilla):
     """Show the floating IP address of the vanilla server."""
 
     def take_action(self, parsed_args):
-        subprocess.check_call("sleep 1 && openstack server show --format value --column addresses %s | cut -d' ' -f 2" % (parsed_args.server), shell=True)
+        self.check_call("openstack server show --format value --column addresses %s | cut -d' ' -f 2" % (parsed_args.server))
         return
 
 ##
 
-class ShowMyIP(command.Command):
+class ShowMyIP(_Vanilla):
     """Show my IP address."""
 
     def take_action(self, parsed_args):
-        subprocess.check_call("sleep 1 && curl --silent https://ipinfo.io/ip", shell=True)
+        self.check_call('curl --silent https://ipinfo.io/ip')
         return
 
 ##
@@ -225,7 +277,7 @@ class ShowStatus(_Vanilla):
     """Show the status of the vanilla server."""
 
     def take_action(self, parsed_args):
-        subprocess.check_call("sleep 1 && openstack server show --format value --column status %s" % (parsed_args.server), shell=True)
+        self.check_call('openstack server show --format value --column status %s' % (parsed_args.server))
         return
 
 ##
@@ -235,18 +287,21 @@ class TakeIP(_Vanilla):
 
     def take_action(self, parsed_args):
         server = parsed_args.server
-        ip = subprocess.check_output("openstack vanilla show ip %s" % (server), shell=True).decode().strip()
+        ip = self.check_output('openstack vanilla show ip %s' % (server)).decode().strip()
         if not re.search('=', ip):
-            subprocess.check_call("sleep 1 && openstack floating ip delete %s" % (ip), shell=True)
+            self.check_call('openstack floating ip delete %s' % (ip))
         return
         
 ##
 
-class Test(_Vanilla, _AddPort):
+class Test(_Vanilla):
     """Test for the plugin."""
 
+    logger = logging.getLogger(__name__)
+
     def take_action(self, parsed_args):
-        print(_add_port_arguments(parsed_args.ports))
+        print(self.app.options.verbose_level)
+        self.check_call('openstack vanilla show ip %s' % (parsed_args.server))
         return
 
 ##
@@ -256,5 +311,9 @@ class Unshelve(_Vanilla, _AddPort):
 
     def take_action(self, parsed_args):
         server = parsed_args.server
-        subprocess.check_call("sleep 1 && openstack server unshelve %s > /dev/null && openstack vanilla allow me %s %s && openstack vanilla give ip %s" % (server, _add_port_arguments(parsed_args.ports), server, server), shell=True)
+        self.check_call('openstack server unshelve %s' % (server), '> /dev/null')
+        self.check_calls([
+            'openstack vanilla allow me %s %s' % (self.add_port_arguments(parsed_args.ports), server),
+            'openstack vanilla give ip %s' % (server)
+        ])
         return
